@@ -12,9 +12,9 @@
 GType
 @enum_name@_get_type (void)
 {
-  static volatile gsize g_define_type_id__volatile = 0;
+  static gsize g_define_type_id_initialized = 0;
 
-  if (g_once_init_enter (&g_define_type_id__volatile))
+  if (g_once_init_enter (&g_define_type_id_initialized))
     {
       static const G@Type@Value values[] = {
 /*** END value-header ***/
@@ -28,10 +28,10 @@ GType
       };
       GType g_define_type_id =
         g_@type@_register_static (g_intern_static_string ("@EnumName@"), values);
-      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+      g_once_init_leave (&g_define_type_id_initialized, g_define_type_id);
     }
 
-  return g_define_type_id__volatile;
+  return g_define_type_id_initialized;
 }
 
 /*** END value-tail ***/
