@@ -166,101 +166,6 @@ load_connection_status (MMBaseBearer        *bearer,
                                    user_data);
 }
 
-/*****************************************************************************/
-/* Auth helpers */
-
-typedef enum {
-    BEARER_CINTERION_AUTH_UNKNOWN   = -1,
-    BEARER_CINTERION_AUTH_NONE      =  0,
-    BEARER_CINTERION_AUTH_PAP       =  1,
-    BEARER_CINTERION_AUTH_CHAP      =  2,
-    BEARER_CINTERION_AUTH_MSCHAPV2  =  3,
-} BearerCinterionAuthType;
-
-static BearerCinterionAuthType
-parse_auth_type (MMBearerAllowedAuth mm_auth)
-{
-    switch (mm_auth) {
-    case MM_BEARER_ALLOWED_AUTH_NONE:
-        return BEARER_CINTERION_AUTH_NONE;
-    case MM_BEARER_ALLOWED_AUTH_PAP:
-        return BEARER_CINTERION_AUTH_PAP;
-    case MM_BEARER_ALLOWED_AUTH_CHAP:
-        return BEARER_CINTERION_AUTH_CHAP;
-    case MM_BEARER_ALLOWED_AUTH_MSCHAPV2:
-        return BEARER_CINTERION_AUTH_MSCHAPV2;
-    case MM_BEARER_ALLOWED_AUTH_UNKNOWN:
-    case MM_BEARER_ALLOWED_AUTH_MSCHAP:
-    case MM_BEARER_ALLOWED_AUTH_EAP:
-    default:
-        return BEARER_CINTERION_AUTH_UNKNOWN;
-    }
-}
-
-/* Cinterion authentication is done with the command AT^SGAUTH,
-   whose syntax depends on the modem family, as follow:
-   - AT^SGAUTH=<cid>[, <auth_type>[, <user>, <passwd>]] for the IMT family
-   - AT^SGAUTH=<cid>[, <auth_type>[, <passwd>, <user>]] for the rest */
-static gchar *
-build_auth_string (MMBroadbandBearerCinterion *self,
-                   MMCinterionModemFamily      modem_family,
-                   MMBearerProperties         *config,
-                   guint                       cid)
-{
-    MMBearerAllowedAuth      auth;
-    BearerCinterionAuthType  encoded_auth = BEARER_CINTERION_AUTH_UNKNOWN;
-    gboolean                 has_user;
-    gboolean                 has_passwd;
-    const gchar             *user;
-    const gchar             *passwd;
-    g_autofree gchar        *quoted_user = NULL;
-    g_autofree gchar        *quoted_passwd = NULL;
-
-    user   = mm_bearer_properties_get_user         (config);
-    passwd = mm_bearer_properties_get_password     (config);
-    auth   = mm_bearer_properties_get_allowed_auth (config);
-
-    has_user     = (user   && user[0]);
-    has_passwd   = (passwd && passwd[0]);
-    encoded_auth = parse_auth_type (auth);
-
-    /* When 'none' requested, we won't require user/password */
-    if (encoded_auth == BEARER_CINTERION_AUTH_NONE) {
-        if (has_user || has_passwd)
-            mm_obj_warn (self, "APN user/password given but 'none' authentication requested");
-        if (modem_family == MM_CINTERION_MODEM_FAMILY_IMT)
-            return g_strdup_printf ("^SGAUTH=%u,%d,\"\",\"\"", cid, encoded_auth);
-        return g_strdup_printf ("^SGAUTH=%u,%d", cid, encoded_auth);
-    }
-
-    /* No explicit auth type requested? */
-    if (encoded_auth == BEARER_CINTERION_AUTH_UNKNOWN) {
-        /* If no user/passwd given, do nothing */
-        if (!has_user && !has_passwd)
-            return NULL;
-
-        /* If user/passwd given, default to PAP */
-        mm_obj_dbg (self, "APN user/password given but no authentication type explicitly requested: defaulting to 'PAP'");
-        encoded_auth = BEARER_CINTERION_AUTH_PAP;
-    }
-
-    quoted_user   = mm_port_serial_at_quote_string (user   ? user   : "");
-    quoted_passwd = mm_port_serial_at_quote_string (passwd ? passwd : "");
-
-    if (modem_family == MM_CINTERION_MODEM_FAMILY_IMT)
-        return g_strdup_printf ("^SGAUTH=%u,%d,%s,%s",
-                                cid,
-                                encoded_auth,
-                                quoted_user,
-                                quoted_passwd);
-
-    return g_strdup_printf ("^SGAUTH=%u,%d,%s,%s",
-                            cid,
-                            encoded_auth,
-                            quoted_passwd,
-                            quoted_user);
-}
-
 /******************************************************************************/
 /* Dial 3GPP */
 
@@ -414,10 +319,10 @@ dial_3gpp_context_step (GTask *task)
     case DIAL_3GPP_CONTEXT_STEP_AUTH: {
         gchar *command;
 
-        command = build_auth_string (self,
-                                     mm_broadband_modem_cinterion_get_family (MM_BROADBAND_MODEM_CINTERION (ctx->modem)),
-                                     mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self)),
-                                     ctx->cid);
+        command = mm_cinterion_build_auth_string (self,
+                                                  mm_broadband_modem_cinterion_get_family (MM_BROADBAND_MODEM_CINTERION (ctx->modem)),
+                                                  mm_base_bearer_peek_config (MM_BASE_BEARER (ctx->self)),
+                                                  ctx->cid);
 
         if (command) {
             mm_obj_dbg (self, "dial step %u/%u: authenticating...", ctx->step, DIAL_3GPP_CONTEXT_STEP_LAST);
@@ -451,7 +356,7 @@ dial_3gpp_context_step (GTask *task)
         mm_base_modem_at_command_full (ctx->modem,
                                        ctx->primary,
                                        command,
-                                       180,
+                                       MM_BASE_BEARER_DEFAULT_CONNECTION_TIMEOUT,
                                        FALSE,
                                        FALSE,
                                        NULL,
@@ -648,7 +553,7 @@ disconnect_3gpp_context_step (GTask *task)
         mm_base_modem_at_command_full (ctx->modem,
                                        ctx->primary,
                                        command,
-                                       10,
+                                       MM_BASE_BEARER_DEFAULT_DISCONNECTION_TIMEOUT,
                                        FALSE,
                                        FALSE,
                                        NULL,
