@@ -101,7 +101,7 @@ create_modem (MMPlugin     *self,
 G_MODULE_EXPORT MMPlugin *
 mm_plugin_create (void)
 {
-    static const gchar *subsystems[] = { "tty", "net", "usb", NULL };
+    static const gchar *subsystems[] = { "tty", "net", "usbmisc", NULL };
     static const guint16 vendor_ids[] = { 0x2cb7, 0 };
     static const gchar *drivers[] = { "cdc_mbim", "qmi_wwan", NULL };
 
